@@ -56,7 +56,7 @@ shift(int *argc, char ***argv)
 void version(void);
 void longversion(void);
 void copyright(void);
-void check_owner_and_repo(const char **owner, const char **repo);
+void check_owner_and_repo(char **owner, char **repo);
 void check_path(struct gcli_path *path);
 
 void parse_labels_options(
@@ -72,6 +72,13 @@ int subcommand_api(int argc, char *argv[]);
 void gcli_pretty_print(char const *input, int indent, int maxlinelen,
                        FILE *stream);
 
-char *gcli_cmd_realpath(char const *const restrict pathname);
+void gcli_pretty_print_diff(char const *const input, int indent);
+
+bool gcli_cmd_should_do_always_yes(void);
+
+void gcli_cmd_save_message(char const *);
+bool gcli_cmd_can_recall_message(void);
+char *gcli_cmd_recall_message(void);
+void gcli_cmd_recall_message_interactive(char **out);
 
 #endif /* GCLI_CMD_CMD_H */
