@@ -33,11 +33,14 @@
 
 #include <gcli/cmd/cmd.h>
 #include <gcli/cmd/cmdconfig.h>
+#include <gcli/cmd/colour.h>
 #include <gcli/port/util.h>
 #include <gcli/repos.h>
 
 #include <ctype.h>
+#include <errno.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <curl/curl.h>
 
@@ -94,7 +97,7 @@ longversion(void)
 }
 
 void
-check_owner_and_repo(const char **owner, const char **repo)
+check_owner_and_repo(char **owner, char **repo)
 {
 	/* HACK */
 	if (gcli_config_get_forge_type(g_clictx) == GCLI_FORGE_BUGZILLA)
@@ -107,7 +110,7 @@ check_owner_and_repo(const char **owner, const char **repo)
 	if (*owner == NULL) {
 		int rc = gcli_config_get_repo(g_clictx, owner, repo);
 		if (rc < 0)
-			errx(1, "gcli: error: %s", gcli_get_error(g_clictx));
+			errx(1, "gcli: error: failed to derive owner/repo combination");
 	}
 }
 
@@ -156,8 +159,8 @@ check_path(struct gcli_path *path)
 	}
 
 	check_owner_and_repo(
-		(char const **)&path->as_default.owner,
-		(char const **)&path->as_default.repo);
+		&path->as_default.owner,
+		&path->as_default.repo);
 }
 
 /* Parses (and updates) the given argument list into two seperate lists:
@@ -335,18 +338,94 @@ gcli_pretty_print(const char *input, int indent, int maxlinelen, FILE *out)
 	}
 }
 
-/* portability kludge for Slowlaris which to this day doesn't support
- * resolved_path to be NULL. */
-char *
-gcli_cmd_realpath(char const *const restrict pathname)
+void
+gcli_pretty_print_diff(char const *const input, int indent)
 {
-	char *resolved_path = NULL;
+	char const *hd = input;
 
-#if defined(_XOPEN_SOURCE) && _XOPEN_SOURCE < 700
-	/* This system is certainly very old! Assume that PATH_MAX is defined.
-	 * If not well there you go. Yes, this is flawed and ugly. */
-	resolved_path = calloc(PATH_MAX, 1);
-#endif
+	for (;;) {
+		char const *eol;
+		char const *start_colour, *end_colour;
+		size_t linelen;
 
-	return realpath(pathname, resolved_path);
+		if (hd == NULL || *hd == '\0')
+			return;
+
+		eol = strchr(hd, '\n');
+		if (eol == NULL)
+			eol = hd + strlen(hd);
+
+		linelen = eol - hd;
+		end_colour = gcli_resetcolour();
+		if (*hd == '+')
+			start_colour = gcli_setcolour(GCLI_COLOR_GREEN);
+		else if (*hd == '-')
+			start_colour = gcli_setcolour(GCLI_COLOR_RED);
+		else
+			start_colour = "";
+
+		printf("%*.*s%s%.*s%s\n", indent, indent, "", start_colour,
+		       (int)linelen, hd, end_colour);
+		hd = eol + 1;
+	}
+}
+
+bool
+gcli_cmd_should_do_always_yes(void)
+{
+	return !isatty(STDIN_FILENO);
+}
+
+void
+gcli_cmd_save_message(char const *const message)
+{
+	FILE *f = fopen("gcli_message", "w");
+	if (!f) {
+		fprintf(stderr, "gcli: warning: failed to open 'gcli_message' "
+		        "for write, cannot save message\n");
+		return;
+	}
+
+	fputs(message, f);
+	fclose(f);
+
+	fprintf(stderr, "gcli: Message was saved in 'gcli_message'. "
+	                "Re-run the command to recall it.\n");
+}
+
+bool
+gcli_cmd_can_recall_message(void)
+{
+	return !access("gcli_message", R_OK);
+}
+
+char *
+gcli_cmd_recall_message(void)
+{
+	char *result = NULL;
+	int rc = 0;
+
+	/* read */
+	rc = gcli_read_file("gcli_message", &result);
+	if (rc < 0)
+		return NULL;
+
+	/* delete old message file */
+	rc = unlink("gcli_message");
+	if (rc < 0) {
+		fprintf(stderr, "gcli: warning: cannot delete gcli_message: %s\n",
+		        strerror(errno));
+	}
+
+	return result;
+}
+
+void
+gcli_cmd_recall_message_interactive(char **out)
+{
+	/* recall an old message if needed, skip if there is a template */
+	if (*out == NULL && gcli_cmd_can_recall_message()) {
+		if (gcli_yesno("Recall previously saved message?"))
+			*out = gcli_cmd_recall_message();
+	}
 }
