@@ -31,23 +31,22 @@
 
 #include <gcli/cmd/cmd.h>
 #include <gcli/cmd/cmdconfig.h>
-#include <gcli/cmd/gitconfig.h>
+#include <gcli/cmd/vcs.h>
 
 #include <gcli/ctx.h>
-#include <gcli/gcli.h>
 #include <gcli/forges.h>
+#include <gcli/gcli.h>
 #include <gcli/gitea/config.h>
 #include <gcli/github/config.h>
 #include <gcli/gitlab/config.h>
 #include <gcli/port/string.h>
 #include <gcli/port/util.h>
+#include <gcli/url.h>
 
 #include <assert.h>
 #include <ctype.h>
 #include <dirent.h>
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#endif
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -56,7 +55,7 @@ struct gcli_config_section {
 
 	struct gcli_config_entries entries;
 
-	gcli_sv title;
+	char *title;
 };
 
 struct gcli_config {
@@ -64,6 +63,7 @@ struct gcli_config {
 
 	char const *override_default_account;
 	char const *override_remote;
+	char const *override_config_file;
 	int override_forgetype;
 	int colours_disabled;       /* NO_COLOR set or output is not a TTY */
 	int force_colours;          /* -c option was given */
@@ -72,8 +72,8 @@ struct gcli_config {
 	int enable_experimental;    /* enable experimental features */
 
 	gcli_sv buffer;
-	char    *file_content;
-	bool    inited;
+	char *file_content;
+	bool inited;
 };
 
 struct gcli_dotgcli {
@@ -113,7 +113,7 @@ should_init_dotgcli(struct gcli_ctx *ctx)
 		(dgcli->has_been_searched_for && !dgcli->has_been_found);
 }
 
-static char const *
+static char *
 find_dotgcli(void)
 {
 	char          *curr_dir_path = NULL;
@@ -160,7 +160,7 @@ find_dotgcli(void)
 
 			free(curr_dir_path);
 
-			curr_dir_path = gcli_cmd_realpath(tmp);
+			curr_dir_path = gcli_realpath(tmp);
 			if (!curr_dir_path)
 				err(1, "gcli: realpath at %s", tmp);
 
@@ -243,8 +243,8 @@ init_local_config(struct gcli_ctx *ctx)
 		struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
 		TAILQ_INSERT_TAIL(&dgcli->entries, entry, next);
 
-	    entry->key = key;
-	    entry->value = value;
+		entry->key = gcli_sv_to_cstr(key);
+		entry->value = gcli_sv_to_cstr(value);
 
 		dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 		curr_line++;
@@ -293,24 +293,33 @@ static void
 parse_section_entry(struct config_parser *input,
                     struct gcli_config_section *section)
 {
-	struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
-	TAILQ_INSERT_TAIL(&section->entries, entry, next);
-
-	gcli_sv key = gcli_sv_chop_until(&input->buffer, '=');
-
-	if (key.length == 0)
-		errx(1, "gcli: %s:%d: empty key", input->filename, input->line);
+	gcli_sv line = gcli_sv_chop_until(&input->buffer, '\n');
+	if (line.length == 0)
+		return;
 
 	input->buffer.data   += 1;
 	input->buffer.length -= 1;
 
-	gcli_sv value = gcli_sv_chop_until(&input->buffer, '\n');
+	gcli_sv key = gcli_sv_chop_until(&line, '=');
+	if (line.length == 0) {
+		errx(1, "gcli: %s:%d: error: missing '=' in config key assignment",
+		     input->filename, input->line);
+	}
 
-	entry->key   = gcli_sv_trim(key);
-	entry->value = gcli_sv_trim(value);
+	line.data   += 1;
+	line.length -= 1;
+
+	if (key.length == 0)
+		errx(1, "gcli: %s:%d: empty key", input->filename, input->line);
+
+	struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
+	TAILQ_INSERT_TAIL(&section->entries, entry, next);
+
+	entry->key= gcli_sv_to_cstr(gcli_sv_trim(key));
+	entry->value = gcli_sv_to_cstr(gcli_sv_trim(line));
 }
 
-static gcli_sv
+static char *
 parse_section_title(struct config_parser *input)
 {
 	size_t len = 0;
@@ -338,7 +347,8 @@ parse_section_title(struct config_parser *input)
 	input->buffer.data   += 1;
 
 	skip_ws_and_comments(input);
-	return title;
+
+	return gcli_sv_to_cstr(title);
 }
 
 static void
@@ -381,40 +391,15 @@ parse_config_file(struct gcli_config *cfg,
 	}
 }
 
-/**
- * Try to load up the local config file if it exists. If we succeed,
- * return 0. Otherwise return -1.
- */
-static struct gcli_config *
-ensure_config(struct gcli_ctx *ctx)
+static void
+read_config_from_file(struct gcli_ctx *ctx, char const *const file_path)
 {
 	struct gcli_config *cfg = ctx_config(ctx);
-	char *file_path = NULL;
 	struct config_parser parser = {0};
 
-	if (cfg->inited)
-		return cfg;
-
-	cfg->inited = true;
-
-	file_path = getenv("XDG_CONFIG_HOME");
-	if (!file_path) {
-		file_path = getenv("HOME");
-		if (!file_path) {
-			gcli_warnx(ctx, "Neither XDG_CONFIG_HOME nor HOME set in env");
-			return cfg;
-		}
-
-		/*
-		 * Code duplication to avoid leaking pointers */
-		file_path = gcli_asprintf("%s/.config/gcli/config", file_path);
-	} else {
-		file_path = gcli_asprintf("%s/gcli/config", file_path);
-	}
-
 	if (access(file_path, R_OK) < 0) {
 		gcli_warn(ctx, "gcli: cannot access config file at %s", file_path);
-		return cfg;
+		return;
 	}
 
 	int len = gcli_read_file(file_path, &cfg->file_content);
@@ -429,8 +414,48 @@ ensure_config(struct gcli_ctx *ctx)
 	parser.filename = file_path;
 
 	parse_config_file(cfg, &parser);
+}
 
-	free((void *)file_path);
+/**
+ * Try to load up the local config file if it exists. If we succeed,
+ * return 0. Otherwise return -1.
+ */
+static struct gcli_config *
+ensure_config(struct gcli_ctx *ctx)
+{
+	struct gcli_config *cfg = ctx_config(ctx);
+	char *file_path = NULL;
+
+	if (cfg->inited)
+		return cfg;
+
+	cfg->inited = true;
+
+	if (cfg->override_config_file) {
+		/* -C command line flag was given */
+		read_config_from_file(ctx, cfg->override_config_file);
+
+	} else {
+		/* autodetect config file */
+		file_path = getenv("XDG_CONFIG_HOME");
+		if (!file_path) {
+			file_path = getenv("HOME");
+			if (!file_path) {
+				gcli_warnx(ctx, "Neither XDG_CONFIG_HOME nor HOME set in env");
+				return cfg;
+			}
+
+			/*
+			 * Code duplication to avoid leaking pointers */
+			file_path = gcli_asprintf("%s/.config/gcli/config", file_path);
+		} else {
+			file_path = gcli_asprintf("%s/gcli/config", file_path);
+		}
+
+		read_config_from_file(ctx, file_path);
+
+		free(file_path);
+	}
 
 	return cfg;
 }
@@ -557,6 +582,10 @@ gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 		  .has_arg = no_argument,
 		  .flag    = NULL,
 		  .val     = 'v' },
+		{ .name    = "config",
+		  .has_arg = required_argument,
+		  .flag    = NULL,
+		  .val     = 'C' },
 		{ .name    = "version",
 		  .has_arg = no_argument,
 		  .flag    = NULL,
@@ -574,7 +603,7 @@ gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 	/* Start off by pre-populating the config structure */
 	readenv(cfg);
 
-	while ((ch = getopt_long(*argc, *argv, "+a:r:cqvt:V", options, NULL)) != -1) {
+	while ((ch = getopt_long(*argc, *argv, "+a:r:cqvt:VC:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'a': {
 			cfg->override_default_account = optarg;
@@ -585,6 +614,9 @@ gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 		case 'c': {
 			cfg->force_colours = 1;
 		} break;
+		case 'C': {
+			cfg->override_config_file = optarg;
+		} break;
 		case 'q': {
 			gcli_setverbosity(ctx, GCLI_VERBOSITY_QUIET);
 		} break;
@@ -642,7 +674,7 @@ find_section(struct gcli_config *cfg, char const *name)
 	struct gcli_config_section *section;
 
 	TAILQ_FOREACH(section, &cfg->sections, next) {
-		if (gcli_sv_eq_to(section->title, name))
+		if (strcmp(section->title, name) == 0)
 			return section;
 	}
 	return NULL;
@@ -663,7 +695,7 @@ gcli_config_get_section_entries(struct gcli_ctx *ctx, char const *section_name)
 		return &s->entries;
 }
 
-gcli_sv
+char const *
 gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
                         char const *key)
 {
@@ -675,55 +707,53 @@ gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
 
 	if (!section) {
 		gcli_warnx(ctx, "gcli: no config section with name '%s'", section_name);
-		return SV_NULL;
+		return NULL;
 	}
 
 	TAILQ_FOREACH(entry, &section->entries, next) {
-		if (gcli_sv_eq_to(entry->key, key))
+		if (strcmp(entry->key, key) == 0)
 			return entry->value;
 	}
 
-	return SV_NULL;
+	return NULL;
 }
 
-static gcli_sv
+static char *
 gcli_local_config_find_by_key(struct gcli_ctx *ctx, char const *const key)
 {
 	struct gcli_dotgcli *lcfg = ctx_dotgcli(ctx);
 	struct gcli_config_entry *entry;
 
 	TAILQ_FOREACH(entry, &lcfg->entries, next) {
-		if (gcli_sv_eq_to(entry->key, key))
+		if (strcmp(entry->key, key) == 0)
 			return entry->value;
 	}
 
-	return SV_NULL;
+	return NULL;
 }
 
-char *
+char const *
 gcli_config_get_editor(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
-	return gcli_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "editor"));
+	return gcli_config_find_by_key(ctx, "defaults", "editor");
 }
 
-char *
+char const *
 gcli_config_get_pager(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
-	return gcli_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "pager"));
+	return gcli_config_find_by_key(ctx, "defaults", "pager");
 }
 
-char *
+char const *
 gcli_config_get_url_open_program(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
-	return gcli_sv_to_cstr(
-		gcli_config_find_by_key(ctx, "defaults", "url-open-program")
-	);
+	return gcli_config_find_by_key(ctx, "defaults", "url-open-program");
 }
 
 static char const *const
@@ -733,24 +763,21 @@ default_account_entry_names[] = {
 	[GCLI_FORGE_GITEA]    = "gitea-default-account",
 	[GCLI_FORGE_BUGZILLA] = "bugzilla-default-account",};
 
-static char *
+static char const *
 get_default_account(struct gcli_ctx *ctx, gcli_forge_type ftype)
 {
 	char const *const defaultname = default_account_entry_names[ftype];
-	gcli_sv act = gcli_config_find_by_key(ctx, "defaults", defaultname);
+	char const *act = gcli_config_find_by_key(ctx, "defaults", defaultname);
 
-	if (!act.length)
-		return NULL;
-
-	return gcli_sv_to_cstr(act);
+	return act;
 }
 
-static char *
+static char const *
 gcli_config_get_account(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg = ctx_config(ctx);
 	gcli_forge_type ftype = gcli_config_get_forge_type(ctx);
-	char *account;
+	char const *account;
 
 	if (cfg->override_default_account) {
 		account = strdup(cfg->override_default_account);
@@ -768,71 +795,54 @@ static char const *const default_urls[] = {
 	[GCLI_FORGE_BUGZILLA] = "https://bugs.freebsd.org/bugzilla",
 };
 
-char *
+char const *
 gcli_config_get_apibase(struct gcli_ctx *ctx)
 {
-	char *acct = gcli_config_get_account(ctx);
-	char *url = NULL;
+	char const *acct = gcli_config_get_account(ctx);
+	char const *url = NULL;
 
 	if (acct) {
-		gcli_sv url_sv = {0};
-
-		url_sv = gcli_config_find_by_key(ctx, acct, "api-base");
+		url = gcli_config_find_by_key(ctx, acct, "api-base");
 
 		/* https://github.com/herrhotzenplotz/gcli/issues/138
 		 *
 		 * Above is correct behaviour. Below is bad/buggy behaviour.
 		 * Check for the second (buggy) option apibase and
 		 * use it if needed. */
-		if (gcli_sv_null(url_sv))
-			url_sv = gcli_config_find_by_key(ctx, acct, "apibase");
-
-		if (!gcli_sv_null(url_sv))
-			url = gcli_sv_to_cstr(url_sv);
+		if (url == NULL)
+			url = gcli_config_find_by_key(ctx, acct, "apibase");
 	}
 
 	if (!url)
-		url = strdup(default_urls[gcli_config_get_forge_type(ctx)]);
-
-
-	free(acct);
+		url = default_urls[gcli_config_get_forge_type(ctx)];
 
 	return url;
 }
 
-char *
+char const *
 gcli_config_get_account_name(struct gcli_ctx *ctx)
 {
-	char *account = gcli_config_get_account(ctx);
-	gcli_sv actname;
+	char const *account = gcli_config_get_account(ctx);
 
 	if (!account)
 		return NULL;
 
-	actname = gcli_config_find_by_key(ctx, account, "account");
-	free(account);
-
-	return gcli_sv_to_cstr(actname);
+	return gcli_config_find_by_key(ctx, account, "account");
 }
 
-static char *
+static char const *
 get_account_token(struct gcli_ctx *ctx)
 {
-	char *account;
-	gcli_sv token;
+	char const *account;
 
 	account = gcli_config_get_account(ctx);
 	if (!account)
 		return NULL;
 
-	token = gcli_config_find_by_key(ctx, account, "token");
-
-	free(account);
-
-	return gcli_sv_to_cstr(token);
+	return gcli_config_find_by_key(ctx, account, "token");
 }
 
-char *
+char const *
 gcli_config_get_token(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
@@ -840,7 +850,7 @@ gcli_config_get_token(struct gcli_ctx *ctx)
 	return get_account_token(ctx);
 }
 
-gcli_sv
+char const *
 gcli_config_get_upstream(struct  gcli_ctx *ctx)
 {
 	init_local_config(ctx);
@@ -851,34 +861,39 @@ gcli_config_get_upstream(struct  gcli_ctx *ctx)
 bool
 gcli_config_pr_inhibit_delete_source_branch(struct gcli_ctx *ctx)
 {
-	gcli_sv val;
+	char const *val;
 
 	init_local_config(ctx);
 
 	val = gcli_local_config_find_by_key(ctx, "pr.inhibit-delete-source-branch");
+	if (val == NULL)
+		return false;
 
-	return gcli_sv_eq_to(val,	"yes");
+	return strcmp(val, "yes") == 0;
 }
 
-void
-gcli_config_get_upstream_parts(struct gcli_ctx *ctx, gcli_sv *const owner,
-                               gcli_sv *const repo)
+static int
+gcli_config_get_upstream_parts(struct gcli_ctx *ctx, char **const owner,
+                               char **const repo)
 {
+	char const *upstream, *hd;
 	ensure_config(ctx);
 
-	gcli_sv upstream = gcli_config_get_upstream(ctx);
-	*owner = gcli_sv_chop_until(&upstream, '/');
+	upstream = gcli_config_get_upstream(ctx);
+	if (upstream == NULL)
+		return -1;
 
-	/* Sanity check: did we actually reach the '/'? */
-	if (*upstream.data != '/')
-		errx(1, "gcli: .gcli has invalid upstream format. expected owner/repo");
+	hd = strrchr(upstream, '/');
+	if (hd == NULL)
+		return -1;
 
-	upstream.data   += 1;
-	upstream.length -= 1;
-	*repo            = upstream;
+	*owner = gcli_strndup(upstream, hd - upstream);
+	*repo = strdup(hd + 1);
+
+	return 0;
 }
 
-gcli_sv
+char const *
 gcli_config_get_base(struct gcli_ctx *ctx)
 {
 	init_local_config(ctx);
@@ -886,7 +901,7 @@ gcli_config_get_base(struct gcli_ctx *ctx)
 	return gcli_local_config_find_by_key(ctx, "pr.base");
 }
 
-gcli_sv
+char const *
 gcli_config_get_override_default_account(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg;
@@ -895,9 +910,9 @@ gcli_config_get_override_default_account(struct gcli_ctx *ctx)
 	cfg = ctx_config(ctx);
 
 	if (cfg->override_default_account)
-		return SV((char *)cfg->override_default_account);
+		return cfg->override_default_account;
 	else
-		return SV_NULL;
+		return NULL;
 }
 
 static gcli_forge_type
@@ -912,12 +927,12 @@ gcli_config_get_forge_type_internal(struct gcli_ctx *ctx)
 	ensure_config(ctx);
 	init_local_config(ctx);
 
-	gcli_sv entry = {0};
+	char const *entry = {0};
 
 	if (cfg->override_default_account) {
-	    char const *section = cfg->override_default_account;
+		char const *section = cfg->override_default_account;
 		entry = gcli_config_find_by_key(ctx, section, "forge-type");
-		if (gcli_sv_null(entry))
+		if (entry == NULL)
 			errx(1,
 			     "gcli: error: given default override account not found or "
 			     "missing forge-type");
@@ -925,21 +940,21 @@ gcli_config_get_forge_type_internal(struct gcli_ctx *ctx)
 		entry = gcli_local_config_find_by_key(ctx, "forge-type");
 	}
 
-	if (!gcli_sv_null(entry)) {
-		if (gcli_sv_eq_to(entry, "github"))
+	if (entry) {
+		if (strcmp(entry, "github") == 0)
 			return GCLI_FORGE_GITHUB;
-		else if (gcli_sv_eq_to(entry, "gitlab"))
+		else if (strcmp(entry, "gitlab") == 0)
 			return GCLI_FORGE_GITLAB;
-		else if (gcli_sv_eq_to(entry, "gitea"))
+		else if (strcmp(entry, "gitea") == 0)
 			return GCLI_FORGE_GITEA;
-		else if (gcli_sv_eq_to(entry, "bugzilla"))
+		else if (strcmp(entry, "bugzilla") == 0)
 			return GCLI_FORGE_BUGZILLA;
 		else
-			errx(1, "gcli: unknown forge type "SV_FMT, SV_ARGS(entry));
+			errx(1, "gcli: unknown forge type %s", entry);
 	}
 
-	/* As a last resort, try to infer from the git remote */
-	int const type = gcli_gitconfig_get_forgetype(ctx, cfg->override_remote);
+	/* As a last resort, try to infer from the VCS */
+	int const type = gcli_cmd_vcs_forgetype(ctx, cfg->override_remote);
 	if (type < 0)
 		errx(1, "gcli: error: cannot infer forge type. "
 		     "use -t <forge-type> to overrride manually.");
@@ -972,11 +987,10 @@ gcli_config_get_forge_type(struct gcli_ctx *ctx)
 }
 
 int
-gcli_config_get_remote(struct gcli_ctx *ctx, char **remote)
+gcli_config_get_remote(struct gcli_ctx *ctx, char const **remote)
 {
 	struct gcli_config *cfg;
 	gcli_forge_type type;
-	int rc;
 
 	cfg = ensure_config(ctx);
 
@@ -986,25 +1000,24 @@ gcli_config_get_remote(struct gcli_ctx *ctx, char **remote)
 	}
 
 	type = gcli_config_get_forge_type(ctx);
-	rc = gcli_gitconfig_get_remote(ctx, type, remote);
 
-	return rc;
+	return gcli_cmd_vcs_remote_by_forgetype(ctx, type, remote);
 }
 
 int
-gcli_config_get_repo(struct gcli_ctx *ctx, char const **const owner,
-                     char const **const repo)
+gcli_config_get_repo(struct gcli_ctx *ctx, char **const owner,
+                     char **const repo)
 {
-	gcli_sv upstream = {0};
 	struct gcli_config *cfg;
+	int rc = 0;
 
 	cfg = ensure_config(ctx);
 
 	if (cfg->override_remote) {
 		int forge = 0, rc = 0;
 
-		rc = gcli_gitconfig_repo_by_remote(ctx, cfg->override_remote, owner,
-		                                   repo, &forge);
+		rc = gcli_cmd_vcs_repo_by_remote(
+			ctx, cfg->override_remote, owner, repo, &forge);
 
 		if (rc < 0)
 			return rc;
@@ -1017,17 +1030,27 @@ gcli_config_get_repo(struct gcli_ctx *ctx, char const **const owner,
 		return 0;
 	}
 
-	if ((upstream = gcli_config_get_upstream(ctx)).length != 0) {
-		gcli_sv const owner_sv = gcli_sv_chop_until(&upstream, '/');
-		gcli_sv const repo_sv = gcli_sv_from_parts(upstream.data + 1, upstream.length - 1);
+	/* attempt a guess via forced forge type */
+	if (cfg->override_forgetype != -1) {
+		char const *rname = NULL;
 
-		*owner = gcli_sv_to_cstr(owner_sv);
-		*repo  = gcli_sv_to_cstr(repo_sv);
+		rc = gcli_cmd_vcs_remote_by_forgetype(
+			ctx, cfg->override_forgetype, &rname);
 
-		return 0;
+		if (rc < 0)
+			return rc;
+
+		rc = gcli_cmd_vcs_repo_by_remote(
+			ctx, rname, owner, repo, NULL);
+
+		return rc;
 	}
 
-	return gcli_gitconfig_repo_by_remote(ctx, NULL, owner, repo, NULL);
+	rc = gcli_config_get_upstream_parts(ctx, owner, repo);
+	if (rc == 0)
+		return 0;
+
+	return gcli_cmd_vcs_repo_by_remote(ctx, NULL, owner, repo, NULL);
 }
 
 bool
@@ -1068,11 +1091,11 @@ gcli_config_display_progress_spinner(struct gcli_ctx *ctx)
 	if (cfg->no_spinner)
 		return false;
 
-	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "disable-spinner");
-	if (gcli_sv_null(cfg_entry))
+	char const *cfg_entry = gcli_config_find_by_key(ctx, "defaults", "disable-spinner");
+	if (cfg_entry == NULL)
 		return true;
 
-	if (string_means_true(gcli_sv_to_cstr(cfg_entry)))
+	if (string_means_true(cfg_entry))
 		return false;
 
 	return true;
@@ -1089,11 +1112,11 @@ gcli_config_render_markdown(struct gcli_ctx *ctx)
 	if (cfg->no_markdown)
 		return false;
 
-	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "render-markdown");
-	if (gcli_sv_null(cfg_entry))
+	char const *cfg_entry = gcli_config_find_by_key(ctx, "defaults", "render-markdown");
+	if (cfg_entry == NULL)
 		return true;
 
-	if (string_means_false(gcli_sv_to_cstr(cfg_entry))) {
+	if (string_means_false(cfg_entry)) {
 		cfg->no_markdown = true;
 		return false;
 	}
@@ -1112,9 +1135,69 @@ gcli_config_enable_experimental(struct gcli_ctx *ctx)
 	if (cfg->enable_experimental)
 		return true;
 
-	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "enable-experimental");
-	if (gcli_sv_null(cfg_entry))
+	char const *cfg_entry = gcli_config_find_by_key(ctx, "defaults", "enable-experimental");
+	if (cfg_entry == NULL)
 		return false;
 
-	return string_means_true(gcli_sv_to_cstr(cfg_entry));
+	return string_means_true(cfg_entry);
+}
+
+static struct gcli_config_entry *
+gcli_config_section_find_entry(struct gcli_config_section *s, char const *entry)
+{
+	struct gcli_config_entry *e;
+
+	TAILQ_FOREACH(e, &s->entries, next) {
+		if (strcmp(e->key, entry) == 0)
+			return e;
+	}
+
+	return NULL;
+}
+
+int
+gcli_config_get_forge_type_by_host(struct gcli_ctx *ctx,
+                                   char const *const host,
+                                   gcli_forge_type *out)
+{
+	struct gcli_config_section *s;
+	struct gcli_config *cfg;
+	int rc = 0;
+
+	cfg = ctx_config(ctx);
+
+	if (host == NULL)
+		return -1;
+
+	TAILQ_FOREACH(s, &cfg->sections, next) {
+		struct gcli_config_entry *e;
+		struct gcli_url url = {0};
+
+		e = gcli_config_section_find_entry(s, "api-base");
+		if (e == NULL)
+			continue;
+
+		rc = gcli_parse_url(e->value, &url);
+		if (rc < 0)
+			continue;
+
+		/* not the right host */
+		if (strcmp(url.host, host) != 0) {
+			gcli_url_free(&url);
+			continue;
+		}
+
+		e = gcli_config_section_find_entry(s, "forge-type");
+		if (e == NULL) {
+			gcli_url_free(&url);
+			continue;
+		}
+
+		rc = gcli_parse_forgetype(ctx, e->value, out);
+
+		gcli_url_free(&url);
+		return rc;
+	}
+
+	return -1;
 }
