@@ -29,22 +29,19 @@
 
 #include <config.h>
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
-
-#include <stdlib.h>
-
 #include <gcli/cmd/cmd.h>
 #include <gcli/cmd/cmdconfig.h>
 #include <gcli/cmd/config.h>
 #include <gcli/cmd/forks.h>
-#include <gcli/cmd/gitconfig.h>
+#include <gcli/cmd/vcs/git.h>
 #include <gcli/cmd/table.h>
 #include <gcli/port/util.h>
 
 #include <gcli/forks.h>
 
+#include <getopt.h>
+#include <stdlib.h>
+
 static void
 usage(void)
 {
@@ -115,7 +112,7 @@ static int
 subcommand_forks_create(int argc, char *argv[])
 {
 	bool always_yes = false;
-	char *in = NULL;
+	char const *in = NULL;
 	int ch;
 	struct gcli_path repo_path = {0};
 
@@ -139,6 +136,8 @@ subcommand_forks_create(int argc, char *argv[])
 		{0},
 	};
 
+	always_yes = gcli_cmd_should_do_always_yes();
+
 	while ((ch = getopt_long(argc, argv, "yo:r:i:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o':
@@ -174,13 +173,16 @@ subcommand_forks_create(int argc, char *argv[])
 	}
 
 	if (!in) {
-		if ((in = gcli_config_get_account_name(g_clictx)) == NULL) {
+		char const *act = gcli_config_get_account_name(g_clictx);
+		if (act == NULL) {
 			errx(1, "gcli: error: could not fetch account: %s",
 			     gcli_get_error(g_clictx));
 		}
+
+		in = act;
 	}
 
-	gcli_gitconfig_add_fork_remote(in, repo_path.as_default.repo);
+	gcli_vcs_git_add_fork_remote(in, repo_path.as_default.repo);
 
 	return EXIT_SUCCESS;
 }
@@ -225,6 +227,8 @@ subcommand_forks(int argc, char *argv[])
 		{0},
 	};
 
+	always_yes = gcli_cmd_should_do_always_yes();
+
 	while ((ch = getopt_long(argc, argv, "n:o:r:ys", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o':
