@@ -37,11 +37,9 @@
 #include <gcli/releases.h>
 #include <gcli/port/util.h>
 
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#endif
-
 #include <stdlib.h>
+#include <string.h>
 
 static void
 usage(void)
@@ -289,6 +287,8 @@ subcommand_releases_create(int argc, char *argv[])
 		{0},
 	};
 
+	always_yes = gcli_cmd_should_do_always_yes();
+
 	while ((ch = getopt_long(argc, argv, "ydpn:t:c:r:o:a:T:",
 	                         options, NULL)) != -1) {
 		switch (ch) {
@@ -407,6 +407,8 @@ subcommand_releases_delete(int argc, char *argv[])
 		{0}
 	};
 
+	always_yes = gcli_cmd_should_do_always_yes();
+
 	while ((ch = getopt_long(argc, argv, "yo:r:", options, NULL)) != -1) {
 		switch (ch) {
 		case 'o':
