@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,10 +34,12 @@
 #include <string.h>
 
 char const *
-gcli_init(gcli_ctx **ctx, gcli_forge_type (*get_forge_type)(gcli_ctx *),
-          char *(*get_token)(gcli_ctx *), char *(*get_apibase)(gcli_ctx *))
+gcli_init(struct gcli_ctx **ctx,
+          gcli_forge_type (*get_forge_type)(struct gcli_ctx *),
+          char const *(*get_token)(struct gcli_ctx *),
+          char const *(*get_apibase)(struct gcli_ctx *))
 {
-	*ctx = calloc(sizeof (struct gcli_ctx), 1);
+	*ctx = calloc(1, sizeof (struct gcli_ctx));
 	if (!(*ctx))
 		return strerror(errno);
 
@@ -45,25 +47,57 @@ gcli_init(gcli_ctx **ctx, gcli_forge_type (*get_forge_type)(gcli_ctx *),
 	(*ctx)->get_token = get_token;
 	(*ctx)->get_apibase = get_apibase;
 
+	(*ctx)->apibase = NULL;
+
 	return NULL;
 }
 
 void
-gcli_destroy(gcli_ctx **ctx)
+gcli_destroy(struct gcli_ctx **ctx)
 {
 	if (ctx && *ctx) {
-		free(*ctx);
-		*ctx = NULL;
+		gcli_clear_ptr(&(*ctx)->apibase);
+
+		gcli_clear_ptr(ctx);
 
 		/* TODO: other deinit stuff? */
 	}
 }
 
 char const *
-gcli_get_error(gcli_ctx *ctx)
+gcli_get_error(struct gcli_ctx *ctx)
 {
 	if (ctx->last_error)
 		return ctx->last_error;
 	else
 		return "No error";
 }
+
+void
+gcli_clear_ptr(void *ptr)
+{
+	void **_ptr = ptr;
+
+	free(*_ptr);
+	*_ptr = NULL;
+}
+
+int
+gcli_parse_forgetype(struct gcli_ctx *ctx, char const *const in,
+                     gcli_forge_type *const out)
+{
+	int rc = 0;
+
+	if (strcmp(in, "github") == 0)
+		*out = GCLI_FORGE_GITHUB;
+	else if (strcmp(in, "gitlab") == 0)
+		*out = GCLI_FORGE_GITLAB;
+	else if (strcmp(in, "gitea") == 0)
+		*out = GCLI_FORGE_GITEA;
+	else if (strcmp(in, "bugzilla") == 0)
+		*out = GCLI_FORGE_BUGZILLA;
+	else
+		rc = gcli_error(ctx, "bad forge type %s", in);
+
+	return rc;
+}
